unit clCreateDatabase;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, forms, sqldb, IBConnection;

  type

  { TCreateDatabase }

 TCreateDatabase = class

  private
    procedure CreateDatabase(aLines: TStrings);
    procedure CreateTables(aLines: TStrings);
    procedure FillTables(aLines: TStrings);
    public
      procedure start;
  end;

implementation

uses dConnections, FCreateDBForm;

procedure TCreateDatabase.start;
var Frm : TFrmCreateDBForm;
begin
  Frm := TFrmCreateDBForm.create(nil);
  try
    Frm.Show;
    application.ProcessMessages;
    sleep(100);
    CreateDatabase(Frm.Memo1.lines);
    Sleep(3000);
    Frm.close;
  finally
    Frm.free;
  end;
end;

procedure TCreateDatabase.CreateDatabase(aLines: TStrings);
begin
    aLines.add('create database.....');
    dmConnections.IBConnection1.CreateDB;
    alines.add('create tables.....');
    CreateTables(aLines);
    dmConnections.IBConnection1.Open;
    alines.Add('fill tables with records....');
    fillTables(aLines);
    alines.add('finished');
end;

procedure TCreateDatabase.FillTables(aLines: TStrings);
var
  SQLQuery       : TSQLQuery;
  SQLTransaction : TSQLTransaction;
begin
  SQLQuery       := TSQLQuery.Create(nil);
  SQLTransaction := TSQLTransaction.create(nil);
  try
    SQLTransaction.DataBase := dmConnections.IBConnection1;
    SQLQuery.DataBase := dmConnections.IBConnection1;
    SQLQuery.Transaction := SQLTransaction;
    SQLQuery.SQL.Text := 'INSERT INTO vat (code, vatvalue) VALUES (:value1, :value2)';
    SQLQuery.Params[0].AsString := 'VAT zero';
    SQLQuery.Params[1].AsFloat := 0;
    SQLQuery.ExecSQL;
    SQLTransaction.commit;
    SQLQuery.SQL.Text := 'INSERT INTO countries (countryname) VALUES (:value1)';
    SQLQuery.Params[0].AsString := 'the Netherlands';
    SQLQuery.ExecSQL;
    SQLTransaction.commit;

    SQLQuery.SQL.Text := 'INSERT INTO defaults (deflable,defvalue) VALUES (:code,:waarde)';
    SQLQuery.Params[0].AsString := 'REPORT';
    SQLQuery.Params[1].AsString := 'factory_factuur.fr3';
    SQLQuery.ExecSQL;
    SQLQuery.Params[0].AsString := 'DESIGNER';
    SQLQuery.Params[1].AsString := 'False';
    SQLQuery.ExecSQL;
    SQLQuery.Params[0].AsString := 'INVOICENO';
    SQLQuery.Params[1].AsString := '1';
    SQLQuery.ExecSQL;
    SQLTransaction.commit;
  finally
    SQLTransaction.free;
    SQLQuery.free;
  end;

end;

procedure TCreateDatabase.CreateTables(aLines: TStrings);
var
  SQLScript      : TSQLScript;
  SQLTransaction : TSQLTransaction;
begin
  SQLScript      := TSQLScript.Create(nil);
  SQLTransaction := TSQLTransaction.create(nil);
  try
    SQLTransaction.DataBase := dmConnections.IBConnection1;
    SQLScript.DataBase := dmConnections.IBConnection1;
    SQLScript.Transaction := SQLTransaction;

    alines.add('Table VAT ');
    SQLScript.Script.Clear;
    SQLScript.Script.Add('  CREATE TABLE VAT (');
    SQLScript.Script.Add('    ID       BIGINT GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('    CODE     VARCHAR(    15) NOT NULL,');
    SQLScript.Script.Add('    VATVALUE DECIMAL(  5, 2) DEFAULT 0 NOT NULL,');
    SQLScript.Script.Add('    CONSTRAINT PK_VAT PRIMARY KEY (ID));');
    SQLScript.Script.Add('CREATE UNIQUE ASC INDEX I_VAT_CODE ON VAT (CODE);');

    SQLScript.ExecuteScript;
    SQLTransaction.Commit;

    SQLScript.Script.Clear;
    alines.add('Table INVOICES');
    SQLScript.Script.Add('CREATE TABLE INVOICES (');
    SQLScript.Script.Add('  ID             BIGINT  GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('  INVOICENUMBER  INTEGER         NOT NULL,');
    SQLScript.Script.Add('  CUSTOMERID        BIGINT         NOT NULL,');
    SQLScript.Script.Add('  INVOICEDATE   DATE         NOT NULL,');
    SQLScript.Script.Add(' CONSTRAINT PK_INVOICED PRIMARY KEY (ID));');
    SQLScript.Script.Add('CREATE UNIQUE ASC INDEX I_INVOICES_NUMBER ON INVOICES (INVOICENUMBER);');
    SQLScript.ExecuteScript;
    SQLTransaction.Commit;

    alines.add('Table INVOICELINES');
    SQLScript.Script.Clear;
    SQLScript.Script.Add('CREATE TABLE INVOICELINES (');
    SQLScript.Script.Add('ID                         BIGINT  GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('INVOICENUMBER              BIGINT         NOT NULL,');
    SQLScript.Script.Add('DESCRIPTION              VARCHAR(    80) NOT NULL,');
    SQLScript.Script.Add('INVOICEAMOUNT             DECIMAL(  8, 2) DEFAULT 0.00 NOT NULL,');
    SQLScript.Script.Add('VATAMOUNT                 DECIMAL(  6, 2) DEFAULT 0.00 NOT NULL,');
    SQLScript.Script.Add('TOTAL                    DECIMAL( 10, 2) DEFAULT 0.00 NOT NULL,');
    SQLScript.Script.Add('VATCODE                   VARCHAR(    15) NOT NULL,');
    SQLScript.Script.Add('CONSTRAINT PK_INVOICELINES PRIMARY KEY (ID));');
    SQLScript.Script.Add('CREATE ASC INDEX I_INVOICELINES_NUMBER ON INVOICELINES (INVOICENUMBER);');
    SQLScript.ExecuteScript;
    SQLTransaction.Commit;

    alines.add('Table COUNTRIES');
    SQLScript.Script.Clear;
    SQLScript.Script.Add('CREATE TABLE COUNTRIES (');
    SQLScript.Script.Add('ID                     BIGINT  GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('COUNTRYNAME            VARCHAR(    25) NOT NULL,');
    SQLScript.Script.Add('CONSTRAINT PK_COUNTRIES PRIMARY KEY (ID));');
    SQLScript.ExecuteScript;
    SQLTransaction.Commit;

    alines.add('Table DEFAULTS');
    SQLScript.Script.Clear;
    SQLScript.Script.Add('CREATE TABLE DEFAULTS (');
    SQLScript.Script.Add('ID      BIGINT  GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('DEFLABLE   VARCHAR(20) NOT NULL,');
    SQLScript.Script.Add('DEFVALUE  VARCHAR(30),');
    SQLScript.Script.Add('CONSTRAINT PK_DEFAULTS PRIMARY KEY (ID));');
    SQLScript.ExecuteScript;
    SQLTransaction.Commit;


    alines.add('Table CUSTOMERS');
    SQLScript.Script.Clear;
    SQLScript.Script.Add('CREATE TABLE CUSTOMERS (');
    SQLScript.Script.Add('ID                       BIGINT   GENERATED BY DEFAULT AS IDENTITY NOT NULL,');
    SQLScript.Script.Add('COMPANY                 VARCHAR(    60) NOT NULL,');
    SQLScript.Script.Add('ATTENTION                VARCHAR(    40),');
    SQLScript.Script.Add('STREET                   VARCHAR(    40),');
    SQLScript.Script.Add('NUMBER                   VARCHAR(     8),');
    SQLScript.Script.Add('ZIPCODE                  VARCHAR(    10),');
    SQLScript.Script.Add('PLACE                    VARCHAR(    30),');
    SQLScript.Script.Add('COUNTRYID                BIGINT,');
    SQLScript.Script.Add('BANKACCOUNT              VARCHAR(    20),');
    SQLScript.Script.Add('MEMBER                   VARCHAR(    20),');
    SQLScript.Script.Add('VATNUMBER                VARCHAR(    25),');
    SQLScript.Script.Add('CONSTRAINT PK_CUSTOMERS  PRIMARY KEY (ID));');
    SQLScript.ExecuteScript;
    SQLTransaction.Commit;

  finally
    SQLTransaction.free;
    SQLScript.free;
  end;
end;

end.

