unit clPrnBaseClass;

{$mode objfpc}{$H+}

interface

uses
  windows,  Classes, SysUtils, DB, clLazMatrixstrings, Bufdataset, SQLdb,
  clLazGlobals, frxClass, frxDesgn, frxDBSet, frxVariables, frxExportODF, math,
  variants, frxExportHTML, clLazFunctions, IBConnection, frxexportPDF, clMyTypes;

type

  { TBasePrClass }

  TBasePrClass = class
  private
    fDesigner: boolean;
    fLdnlijstaanvulling: string;
    fRapportscript   : string;

    function DBGetRapport(const aName: string): string;
    function Vertaling(Query: Tdataset; const aValue: string): string;
  protected
      fConn            : TIBConnection;
      frxDesigner      : TfrxDesigner;
      frxHTMLExport1   : TfrxHTMLExport;
      frxODSExport1    : TfrxODSExport;
      frxODTExport1    : TfrxODTExport;
      frxReport        : TfrxReport;

      fExportKeuze     : integer;
      fBlanco          : boolean;
      fBlancoAantal    : integer;
      FPrintOpdracht   : string;
      // extern classes
      Matrix           : TMatrixString;
//      PrintDevice      : TPrinterDevice;
      FHeaderTable     : TBufdataset;
      FMatrixTable     : TBufdataset;
      FWinprnfile      : TStringlist;
      FDefault         : TStringList;
      FFloatStrings    : TStringList;
      FExtendsFields   : TStringList;
      PDFFile          : string;
      FScriptname      : string;
      fExport_Enabled  : boolean;
  public
    fBsKooiQuery       : Tstrings;
    FConverteerTekst   : boolean;
    FPrnError          : integer;
    FZoekpaden         : TSearchpath;
    FRptVariables      : TfrxVariables;
    FslMatrix          : TStringlist;
    FslVelden          : Tstringlist;
    FMatrixFile        : string;
    FFilePath          : string;
    FProgrid           : string;
    FDatafolder        : string;

    constructor create;
    destructor  Destroy; override;

    procedure frxAdd(const aName, aValue: string);
    procedure AssignRptVars;
    procedure LoadMatrix;
    procedure CutIntoPieces(const value: string; var woord: array of string);
//    procedure Basis;
    procedure SetWinprnfilesettings(const aValue : string);
    procedure LoadReport(const aDirectPrint : boolean; const FRXNaam : string = 'DBKooiindeling');

    function DBGetKnuffelkeuring: integer;
    function RPTDataSet(const ParIntNummer: word): TBufdataset;
    function GetRPTDatasets : TArray_of_dataset;
    function MTDataSet : TBufdataset;
    function RPTDataSetActive(const ParIntNummer: word): boolean;
    function RPTFormat(ParValue : string; const ParDefault: string = '*') : string;
    function SetDate(const Value: string): TDate;
    function IsFloatString(const value: string): boolean;
    function PrnError : boolean;
    function DefValue(const value : string; const default: string = 'N'): string;
    function HeaderToTable(const FileName : string; out Error : byte;
                                    const printcodes : boolean = true ) : boolean;
    function SetInt(const Value: string): integer;
    function SetDecimal(const Value: string): single;
    function CreateHeaderTable: boolean;

    property Rapportscript      : string read fRapportscript write fRapportscript;
    property Ldnlijstaanvulling : string read fLdnlijstaanvulling write fLdnlijstaanvulling;
    property Connection         : TIBConnection read fConn write fConn;
    property Desinger           : boolean read fDesigner write fDesigner;
end;


implementation

constructor TBasePrClass.create;
begin
  Fslmatrix     := TStringlist.Create;
  FslVelden     := Tstringlist.create;
  FRptVariables := TfrxVariables.create;
  inherited Create;
end;

destructor TBasePrClass.Destroy;
begin
  FslMatrix.Free;
  FslVelden.Free;
  FRptVariables.free;
  inherited destroy;
end;

procedure TBasePrClass.AssignRptVars;
begin
  frxreport.Variables.Clear;
  frxreport.Variables.Assign(FRptVariables);
end;

procedure TBasePrClass.LoadMatrix;
begin
  FslMatrix.LoadFromFile(FDatafolder + '\' + FMatrixFile);
end;

function TBasePrClass.PrnError: boolean;
begin
  result := FPrnError > 0;
end;

function TBasePrClass.RPTDataSet(const ParIntNummer: word): TBufdataset;
begin
  result := Matrix.Dataset(ParIntNummer);
end;

function TBasePrClass.GetRPTDatasets: TArray_of_dataset;
begin
  setlength(result,6);
  result[0] := RPTDataSet(0);
  result[1] := RPTDataSet(1);
  result[2] := RPTDataSet(2);
  result[3] := RPTDataSet(3);
  result[4] := RPTDataSet(4);
  result[5] := RPTDataSet(5);
end;

function TBasePrClass.MTDataSet: TBufdataset;
begin
  result := FMatrixTable;
end;

function TBasePrClass.RPTDataSetActive(const ParIntNummer: word): boolean;
begin
   result := Matrix.DataSourceLength(ParIntNummer) > 0;
end;

function TBasePrClass.DefValue(const value : string; const default : string = 'N') : string;
begin
  if length(value) > 0 then
    result := value
  else
    result := default;

end;

function TBasePrClass.RPTFormat(ParValue : string; const ParDefault: string = '*'): string;
begin
  if (ParValue = '') then
    result := quotedstr(ParDefault)
  else
    result := quotedstr(ParValue)
end;

procedure TBasePrClass.SetWinprnfilesettings(const aValue: string);
begin
  FWinprnFile.Add(aValue);
end;

function TBasePrClass.CreateHeaderTable : boolean;
begin
  result := true;
  try
    with FHeaderTable do
    begin
      FieldDefs.Add('veldnaam',ftString, 40);
      FieldDefs.Add('veldgrootte',ftInteger, 0);
      FieldDefs.Add('matrixnummer',ftInteger, 0);
      FieldDefs.Add('datatype',ftString, 1);
    end;
    FHeaderTable.Active := true;
  except
    result := false;
  end;
end;



procedure TBasePrClass.CutIntoPieces(const value : string; var woord : array of string);
var teller, teller2 : word;
    scheidingsteken : word;
begin
  scheidingsteken := pos(';',value);
  woord[1] := '80';
  woord[2] := '1';
  woord[3] := 'S';
  if scheidingsteken = 0 then
  begin
    woord[0] := value;
  end else
  begin
    teller2 := 0;
    woord[0] := '';
    for teller := 1 to length(value) do
    begin
      if value[teller] = ';' then
      begin
        inc(teller2);
        woord[teller2] := '';
      end
      else woord[teller2] := woord[teller2] + value[teller];
    end;
  end;
end;

{
procedure TBasePrClass.Basis;
begin
  FZoekpaden.Basis   := SetDefaultPath(uGlobal.GetVar('DEF_STANDAARD'));
  FZoekpaden.Report  := SetCorrectPath(FZoekpaden.Basis, uGlobal.GetVar('DEF_RAPPORT'));
  FRptVariables.Clear;//clear list
end;
}

function TBasePrClass.HeaderToTable(const FileName : string; out Error : byte;
                                    const printcodes : boolean = true) : boolean;
var slHeader: TStringList;
    slHeaderControle : TStringList;
    slMatrix: TStringList;
    slMatrixRegel: TStringList;
    teller,
    woordteller,veldteller  : integer;
    woord   : array[0..3] of string;
    folder, S  : string;
    firstletter : string;
    FTextFile : TextFile;
//    ch        : char;

    Tekstregel : string;
    tekstvrij  : string;
begin
  slHeader := TStringList.Create;
  slHeaderControle := TStringList.Create;
  slMatrix := TStringList.Create;
  slMatrixRegel := TStringList.Create;
  slMatrixRegel.Delimiter := '|';
  slMatrixRegel.StrictDelimiter := true;
  result := true;
  Error := 0;
  if FHeaderTable.RecordCount > 0 then
    FHeaderTable.active := false;
  try
    if FileExists(Filename) then
    begin
      slHeader.LoadFromFile(FileName);
      folder := ExtractFilePath(FileName);
      for teller := 0 to slHeader.Count - 1 do
      begin
        CutIntoPieces(slHeader[teller], woord);
        slHeaderControle.Add(woord[0]);
        if not (copy(woord[0],1,1) = '_') then
          FHeaderTable.AppendRecord([woord[0],StrToIntDef(woord[1],80),
            StrToIntDef(woord[2],1),woord[3]]);
      end;
      FHeaderTable.First;
      while not FHeaderTable.Eof do
      begin
        if not (copy(FHeaderTable.FieldByName('veldnaam').AsString,1,1) = '_') then
          FMatrixTable.Fielddefs.Add(FHeaderTable.FieldByName('veldnaam').AsString,
                                ftString,
                                FHeaderTable.FieldByName('veldgrootte').AsInteger);
        FHeaderTable.Next;
      end;
      FMatrixTable.Active := true;
      if FileExists(FZoekpaden.Matrix + FPrintOpdracht) then
      begin
        AssignFile(FTextFile,FZoekpaden.Matrix + FPrintOpdracht);
        Reset(FTextFile);
        try
          Tekstregel := '';
          while not eof(FTextfile) do
          begin
            slMatrixRegel.Clear;
            readln(FTextfile,tekstregel);
            tekstregel := stringreplace(tekstregel,'"','',[rfReplaceAll]);
            slMatrixRegel.QuoteChar := '"';
            slMatrixRegel.DelimitedText := tekstregel;
            FMatrixTable.Append;
            veldteller := 0;
            for woordteller := 0 to slMatrixRegel.Count - 1 do
            begin
              S := slMatrixRegel[woordteller];
              if Printcodes then
              begin
                firstletter := slHeaderControle[veldteller];
                if not ((woordteller >= 1) and (woordteller <= 42))  then
                begin
                  if not (copy(firstletter,1,1) = '_') then
                  begin
                    if FMatrixTable.Fields[veldteller].FieldName = 'vrij_tekst' then
                    begin
                      tekstvrij := stringreplace(slMatrixRegel[woordteller],'~',#10,[rfReplaceAll]);
                      FMatrixTable.Fields[veldteller].Value := tekstvrij;
                    end else
                      FMatrixTable.Fields[veldteller].Value := slMatrixRegel[woordteller];
                  end;
                  inc(veldteller);
                end;
              end else
              begin
                firstletter := slHeaderControle[woordteller];
                if not (copy(slHeaderControle[woordteller],1,1) = '_') then
                begin
                  FMatrixTable.Fields[veldteller].Value := slMatrixRegel[woordteller];
                  inc(veldteller);
                end;
              end;
            end;
            FMatrixTable.Post;
          end;
        except
          result := false;
          Error := 3;
        end;
        CloseFile(FTextfile);
      end else
      begin
        result := false;
        Error := 2;
      end;
    end else
    begin
      result := false;
      Error := 1;
    end;
  finally
    slHeader.Free;
    slHeaderControle.Free;
    slMatrix.free;
    slMatrixRegel.Free;
  end;
end;


function TBasePrClass.SetInt(const Value : string) : integer;
var FloatValue: Single;
    FormatSettings: TFormatSettings;
begin
  GetLocaleFormatSettings(LOCALE_SYSTEM_DEFAULT, FormatSettings);
  FormatSettings.DecimalSeparator := '.';
  FormatSettings.ThousandSeparator := ',';
  FloatValue := StrToFloat(Value, FormatSettings);
  result := trunc(FloatValue);
end;

function TBasePrClass.SetDecimal(const Value : string) : single;
var
  FloatValue: Single;
  FormatSettings: TFormatSettings;
begin
  GetLocaleFormatSettings(LOCALE_SYSTEM_DEFAULT, FormatSettings);
  FormatSettings.DecimalSeparator := '.';
  FormatSettings.ThousandSeparator := ',';
  TryStrToFloat(Value, FloatValue, FormatSettings);
  result := roundto(FloatValue,-2);
end;


function TBasePrClass.Vertaling(Query : Tdataset; const aValue : string) : string;
begin
  if Query.Locate('tekstlabel',vararrayof([uppercase(aValue)]),[]) then
    result := Query.FieldByName('vertaling').AsString
  else
    result := aValue;
end;

function TBasePrClass.SetDate(const Value : string) : TDate;
var dag, maand, jaar : word;
begin
  if Value > '' then
  begin
    dag := strtoint(copy(Value,1,2));
    maand := strtoint(copy(Value,4,2));
    if length(trim(Value)) = 10 then
      jaar := strtoint(copy(Value,7,4))
    else
    begin
      jaar := strtoint(copy(Value,7,2));
      if (jaar > 80) and (jaar  <= 99) then
        jaar := jaar + 1900
      else
        jaar := jaar + 2000;
    end;
    result := encodeDate(jaar,maand,dag);
  end else
    result := 0;
end;

function TBasePrClass.IsFloatString(const value : string) : boolean;
begin
  result := FFloatStrings.IndexOf(value) > -1;
end;

function TBasePrClass.DBGetKnuffelkeuring : integer;
var Query    : TSQLQuery;
    Trans    : TSQLTransAction;
    volgorde : integer;
begin
  Query := TSQLQuery.create(nil);
  Trans := TSQLTransAction.Create(Query);
  try
    Trans.DataBase := fConn;
    Query.DataBase := fConn;
    Query.Transaction := Trans;
    Query.SQL.Text := 'SELECT max(keurvolgorde) FROM soortkeuring';
    Query.Active := true;
    volgorde := Query.fields[0].AsInteger;
    Query.active := false;
    Query.SQL.Text := 'SELECT id FROM soortkeuring WHERE keurvolgorde = :id';
    Query.Params[0].AsInteger := volgorde;
    Query.Active := true;
    result := Query.fields[0].AsInteger;
    Trans.Commit;
  finally
    Query.free;
  end;
end;


function TBasePrClass.DBGetRapport(const aName : string) : string;
var Query    : TSQLQuery;
    Trans    : TSQLTransAction;
begin
  Query  := TSQLQuery.create(nil);
  Trans  := TSQLTransAction.Create(Query);
  result := '';
  try
    Trans.DataBase := fConn;
    Query.DataBase := fConn;
    Query.Transaction := Trans;
    Query.SQL.Text := 'SELECT rapportscript FROM rapportlijsten WHERE label = :id';
    QUery.params[0].AsString := aName;
    Query.Active := true;
    result := Query.fields[0].AsString;
    Trans.Commit;
  finally
    Query.free;
  end;
end;

procedure TBasePrClass.frxAdd(const aName, aValue: string);
begin
  FRptVariables[aName] := quotedStr(aValue);
end;

procedure TBasePrClass.LoadReport(const aDirectPrint : boolean; const FRXNaam : string = 'DBKooiindeling');
var //Designer       : boolean;
    script         : string;
    frxDBDataset   : TfrxDBDataset;
begin
  frxReport      := TfrxReport.create(nil);
  frxDBDataset   := TfrxDBDataset.create(nil);
  frxDesigner    := TfrxDesigner.create(nil);
//  frxpdfexport   := TfrxPDFExport.Create(frxReport);
  try
    frxDBDataset.DataSet  := Matrix.Dataset(1);
    frxDBDataset.UserName := 'invoice';
//    Designer := true; //uGlobal.GetVar('DEF_DESIGNER') = 'J';
    frxReport.Clear;
    script := FRXNaam;
    frxReport.LoadFromFile(script);
    if fDesigner then
      frxReport.DesignReport
    else
    begin
      frxReport.PrepareReport;
      if aDirectPrint then
      begin
        frxReport.PrintOptions.ShowDialog := false;
        frxReport.previewpages.Print;
      end else
        frxReport.ShowPreparedReport;
    end;
  finally
    frxDBDataset.free;
    frxDesigner.free;
    frxReport.free;
  end;
end;

end.

