unit dlookups;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, sqldb, db, variants, dConnections, DBCtrls,
  dialogs;

type

  { TDMLookups }

  TDMLookups = class(TDataModule)
    ListSrcCountries: TDatasource;
    ListSrcVAT: TDataSource;
    L_Countries: TSQLQuery;
    L_VAT: TSQLQuery;
    procedure DataModuleDestroy(Sender: TObject);
  private
  public
    function Lookupvalue(const aLookuptable, aField, aResult : string;
                         const aVariant : variant): variant;
    function CreateDefaultTransaction(Sender: TComponent): TSQLTransaction;

    procedure Connect(Combo: TDBLookupComboBox; Listname: string;
      const OnlyConnect: boolean=false);
    procedure Link(Combo: TDBLookupComboBox; Lsource: TDataSource);
    procedure Open(lookuptable: string);
    { private declarations }
  public
    { public declarations }
  end;

implementation

{$R *.lfm}

{ TDMLookups }

procedure TDMLookups.DataModuleDestroy(Sender: TObject);
var teller  : integer;
    MyQuery : TSQLQuery;
begin
  for teller := 0 to componentcount - 1 do
  begin
    if components[teller] is TSQLQuery then
    begin
      MyQuery := TSQLQuery(components[teller]);
      if MyQuery.Active then
      begin
        if MyQuery.State in [dsInsert, dsEdit] then
          MyQuery.Cancel;
        MyQuery.Active := false;
      end;
    end;
  end;
end;

procedure TDMLookups.Connect(Combo : TDBLookupComboBox; Listname : string;
                            const OnlyConnect : boolean);
var Lsource  : TDatasource;
    SQLQuery : TSQLQuery;
begin
  Lsource := TDatasource(FindComponent(Listname));
  if assigned(LSource) then
  begin
    SQLQuery             := TSQLQuery(Lsource.dataset);
    SQLQuery.Transaction := CreateDefaultTransaction(SQLQuery);
    SQLQuery.database    := DMConnections.DBConnection;
    SQLQuery.Active      := true;
    if not OnlyConnect then
      Combo.ListSource := Lsource;
  end else
    showmessage('[Connect] Kan datasource niet vinden');
end;

procedure TDMLookups.Link(Combo : TDBLookupComboBox; Lsource : TDataSource);
begin
  Combo.ListSource := Lsource;
end;

procedure TDMLookups.Open(lookuptable : string);
var Myquery : TSQLQuery;
begin
  Myquery := TSQLQuery(FindComponent(lookuptable));
  if assigned(MyQuery) then
  begin
    MyQuery.Transaction := CreateDefaultTransaction(MyQuery);
    MyQuery.database := DMConnections.DBConnection;
    MyQuery.Active := true;

  end;
end;

function TDMLookups.Lookupvalue(const aLookuptable, aField, aResult: string;
                                const aVariant : variant): variant;
var Myquery : TSQLQuery;
begin
  try
  Myquery := TSQLQuery(FindComponent(alookuptable));
  if assigned(MyQuery) then
    if MyQuery.Locate(aField,vararrayof([aVariant]),[]) then
      result := MyQuery.FieldByName(aResult).Value
    else
      result := ''
  except
    result := ''
  end;
end;

function TDMLookups.CreateDefaultTransaction(Sender: TComponent): TSQLTransaction;
var Trans: TSQLTransaction;
begin
  Trans := TSQLTransaction.Create(Sender);
  Trans.Database := DMConnections.DBConnection;
  Trans.Action := caNone;
  Trans.Params.Add('read_committed');
  Trans.Params.Add('rec_version');
  Trans.Params.Add('nowait');
  Trans.Active := false;
  Result := Trans;
end;

end.

