unit clBase;

interface

uses classes, variants, sysutils, DB, SQLdb, DBase, dialogs, clLazFunctions;

type

{ TBase }

 TBase = class
    FBaseModule       : TDMBase;
    FCurrentDataset   : TDataset;
    fBasisQuery       : string;
    fCurrentTablename : string;
    fTablename        : string;
    fRequiredfields   : array of string;

private
   protected

  public
    constructor Create; virtual;

    procedure CreateDMModule(DMClass : TDMBaseLink);
    procedure QueryClose(const tablename: string);
    procedure QueryOpen(const tablename: string);
    procedure QueryRefresh(const tablename: string = '');
    procedure RecordNieuw(const tablename: string = '');
    procedure RecordPost(const tablename: string = '');
    procedure RecordPrior(const tablename: string = '');
    procedure RecordNext(const tablename: string = '');
    procedure RecordCancel(const tablename: string = '');
    procedure RecordDelete(const tablename: string = '');
    procedure RecordFirst(const tablename: string = '');
    procedure RecordLast(const tablename: string = '');

    procedure Sort(const aFieldname : string);

    procedure StartFind;

    procedure SetRequiredField(const aValue : string);
    procedure RecordNew(const tablename: string = '');


    //function QueryTable(const tablename : string = '') : TMyQuery;
    function GetDataset(Dataset : TDataset) : TDataset; overload;
    function GetDataset(const tabelnaam : string = '') : TDataset; overload;
    function RecordCount(const tablename: string = '') : integer;
    function RecordNo(const tablename: string = '') : integer;
    function RecordValue(const fieldname : string; const tablename : string = '') : TField;
    function RecordsEmpty: boolean;

    function  Requiredfields : boolean;

    property Datamodule       : TDMBase read FBasemodule;
    property BasisQuery       : string read fBasisQuery write fBasisQuery;
    property Tablename        : string read fTablename write fTablename;
    property CurrentTableName : string read fCurrentTableName write fCurrentTableName;

end;

type TBaseLink = class of TBase;

implementation

constructor TBase.Create;
begin
  inherited;
  CreateDMModule(TDMBase)
end;

procedure TBase.CreateDMModule(DMClass: TDMBaseLink);
begin
  if DMClass <>nil then
  begin
    FBasemodule.free;
    FBaseModule := DMClass.create(nil);
  end;
end;

procedure TBase.StartFind;
begin
  StartfindScreen(fcurrentdataset);
end;


procedure TBase.SetRequiredField(const aValue: string);
var teller : integer;
begin
  teller := length(fRequiredfields);
  setlength(fRequiredfields,teller + 1);
  fRequiredfields[teller] := aValue
end;

function TBase.Requiredfields: boolean;
var index : integer;
begin
  result := true;
  for index := low(fRequiredfields) to high(fRequiredfields) do
  begin
    if recordvalue(fRequiredfields[index]).IsNull then
    begin
      result := false;
      showmessage(format('Veld %s mag niet leeg zijn',[fRequiredfields[index]]) )
    end;
  end;
end;

procedure TBase.QueryClose(const tablename: string);
begin
  Getdataset.Active := false
end;

procedure TBase.QueryOpen(const tablename: string);
begin
  Getdataset.Active := true
end;

procedure TBase.QueryRefresh(const tablename: string);
begin
  GetDataset.Refresh
end;

procedure TBase.RecordNieuw(const tablename: string);
begin
  if fcurrentdataset.state in [dsBrowse] then
    GetDataset.Append;
end;

function TBase.GetDataset(Dataset: TDataset): TDataset;
begin
  result := Dataset
end;

function TBase.GetDataset(const tabelnaam: string): TDataset;
begin
  result := nil;
  if FBaseModule <> nil then
  //if assigned(FBaseModule) then
  begin
    if tabelnaam = '' then
      result := fCurrentdataset
    else
      result := Getdataset(Datamodule.FindComponent(tabelnaam) as TDataset)
  end;
end;

procedure TBase.RecordCancel(const tablename: string);
begin
  GetDataset(tablename).Cancel
end;

function TBase.RecordCount(const tablename: string): integer;
begin
  result := GetDataset(tablename).RecordCount
end;

procedure TBase.RecordDelete(const tablename: string);
begin
  Getdataset(tablename).Delete
end;

procedure TBase.RecordNext(const tablename : string);
begin
  GetDataset(tablename).Next
end;

procedure TBase.RecordFirst(const tablename : string);
begin
  GetDataset(tablename).First
end;

procedure TBase.RecordLast(const tablename : string);
begin
  GetDataset(tablename).Last
end;

procedure TBase.Sort(const aFieldname: string);
begin
  if fcurrentdataset.state in [dsBrowse] then
  begin
    datamodule.DBSort(aFieldname, GetDataset)
//    Query := Getdataset as TSQLQuery;
//    Query.indexfieldnames := aFieldName;
  end;
end;

procedure TBase.RecordNew(const tablename: string);
begin
  if fcurrentdataset.state in [dsBrowse] then
    GetDataset.Append;
end;

function TBase.RecordNo(const tablename: string): integer;
begin
  result := GetDataset(tablename).RecNo
end;

procedure TBase.RecordPost(const tablename: string);
begin
  GetDataset(tablename).Post
end;

procedure TBase.RecordPrior(const tablename : string);
begin
  GetDataset(tablename).Prior
end;

function TBase.RecordValue(const fieldname: string; const tablename: string
  ): TField;
begin
  result := GetDataset(tablename).FieldByName(fieldname)
end;


function TBase.RecordsEmpty : boolean;
begin
  result := false;
  if FCurrentDataset.recordcount = 0 then
  begin
    result := true;
    RecordNieuw;
  end;
end;


end.
