unit clInvoicelines;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DB, clBase, dInvoices, clLazGlobals, DBCtrls, stdCtrls,
  dlookups, Forms, dialogs;

type

{ TInvoiceLines }

 TInvoiceLines = class(TBase)
 private
   FDMLookups : TDMLookups;
   function GetModule: TDMInvoices;

  public
    constructor Create; override;
    destructor  Destroy; override;

    procedure LookupConnect(Combobox: TDBLookupComboBox; const SourceName: string);
    procedure SetTotal(const aSwithIndex : smallint; const aVATcode : string = '');
    procedure SaveLine;

    function Init(Ds1 : TDatasource; aFactuurnr : TLabel) : boolean;

    property Datamodule : TDMInvoices read GetModule;
end;

implementation

{ TInvoiceLines }

constructor TInvoiceLines.Create;
begin
  inherited Create;
  CreateDMModule(TDMInvoices);
  FDMLookups := TDMLookups.create(nil);
end;

destructor TInvoiceLines.Destroy;
begin
  fcurrentdataset.Active := false;
  FDMLookups.free;  // <-- dit wordt eerder uitgevoerd, waardoor mijn andere tabllen worden afgesloten
  datamodule.free;
  inherited Destroy;
end;

function TInvoiceLines.Init(Ds1: TDatasource; aFactuurnr : TLabel): boolean;
begin
  aFactuurnr.caption := IntToStr(uGlobal.GetvarInt('INVOICENO'));
  fCurrentdataset    := datamodule.SQLInvoiceLinesEdit ;
  datamodule.SQLInvoiceLinesEdit.Transaction := datamodule.TrInvoiceLinesEdit;
  if fCurrentdataset <> nil then
  begin
    datamodule.SQLInvoiceLinesEdit.Params[0].AsInteger := uGlobal.GetvarInt('INVOICENO');
    Ds1.dataset            := fCurrentdataset;
    fCurrentdataset.Active := true;
    fcurrentdataset.FieldByName('ID').Required := false;
    result := true;
  end else
    result := false;
end;

function TInvoiceLines.GetModule: TDMInvoices;
begin
  result := TDMInvoices(inherited datamodule)
end;

procedure TInvoiceLines.LookupConnect(Combobox : TDBLookupComboBox; const SourceName : string);
begin
  FDMLookups.Connect(ComboBox,SourceName);
end;

procedure TInvoiceLines.SetTotal(const aSwithIndex : smallint; const aVATcode : string = '');
var vatvalue : variant;
begin
  if fcurrentdataset.State in [dsInsert, dsEdit] then
  case aSwithIndex of
    1 : fcurrentdataset.FieldByName('total').AsFloat := fcurrentdataset.FieldByName('invoiceamount').AsFloat;
    2 : begin
          vatvalue := FDMLookups.Lookupvalue('l_vat','code','vatvalue',aVATcode);
          if vatvalue > 0 then
            fcurrentdataset.FieldByName('vatamount').AsFloat :=
              (fcurrentdataset.FieldByName('invoiceamount').AsFloat * vatvalue) / 100
          else
            fcurrentdataset.FieldByName('vatamount').AsFloat := 0.00;
          fcurrentdataset.FieldByName('total').AsFloat :=
             fcurrentdataset.FieldByName('invoiceamount').AsFloat +
             fcurrentdataset.FieldByName('vatamount').AsFloat;
        end;
    3 : fcurrentdataset.FieldByName('total').AsFloat :=
           fcurrentdataset.FieldByName('invoiceamount').AsFloat +
           fcurrentdataset.FieldByName('vatamount').AsFloat;
  end;
end;

procedure TInvoiceLines.SaveLine;
begin
  if fCurrentdataset.State in  [dsInsert] then
    fcurrentdataset.FieldByName('invoicenumber').AsInteger := uGlobal.GetvarInt('INVOICENO');
  fcurrentdataset.Post;
end;

end.

