unit clInvoicepr;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, clPrnBaseClass, SQLdb, IBConnection, clLazMatrixstrings,
  db, variants;

type

  { TInvoicepr }

  TInvoicepr = class(TBasePrClass)
  private
    fID : integer;
  protected
  public
    constructor create(aConn : TIBConnection; const aExportKeuze: integer; const aRapportkeuze : string);
    destructor  Destroy; override;

    procedure Prepare(const id : integer);
    procedure CreateBody ;
    procedure Finish     ;

end;


implementation

{ TInvoicepr }

constructor TInvoicepr.create(aConn : TIBConnection; const aExportKeuze: integer; const aRapportkeuze : string);
begin
  fConn        := aConn;
  fExportKeuze := aExportKeuze;
  Rapportscript:= aRapportkeuze;
  inherited create;
end;

destructor TInvoicepr.Destroy;
begin
  Matrix.free;
  inherited Destroy;
end;

procedure TInvoicepr.Prepare(const id : integer);
var MatrixOpen : boolean;
begin
  Matrix := TMatrixString.Create;
//  Basis;
  MatrixOpen := Matrix.CreateMatrixFields([
  'ID|10|N',
  'INVOICENUMBER|10|N',
  'INVOICEDATE|12|D',
  'DESCRIPTION|70|S',
  'AMOUNT|10|F',
  'VATAMOUNT|10|F',
  'VATCODE|15|S',
  'INVOICEID|10|N',
  'TOTAL|10|F',
  'MEMBER|15|S',
  'COMPANY|60|S',
  'ATTENTION|30|S',
  'STREET|40|S',
  'NUMBER|10|S',
  'PLACE|30|S',
  'ZIPCODE|15|S',
  'COUNTRYNAME|30|S',
  'VATNUMBER|20|S',
  'VALUE|10|F'],1, false);
  fID := id;
end;

procedure TInvoicepr.CreateBody;
var SQLQuery     : TSQLQuery;
    Transaction  : TSQLTransaction;
begin
  SQLQuery     := TSQLQuery.create(nil);
  Transaction  := TSQLTransaction.Create(SQLQuery);
  try
    SQLQuery.DataBase        := fConn;
    Transaction.DataBase     := fConn;
    SQLQuery.Transaction     := Transaction;
    with SQLQuery.SQL do
    begin
      add('SELECT F.ID, F.INVOICENUMBER, F.INVOICEDATE,  FR.DESCRIPTION, FR.INVOICEAMOUNT, FR.VATAMOUNT,');
      add('       FR.VATCODE, FR.TOTAL,');
      add('       N.MEMBER, N.COMPANY, N.ATTENTION, N.STREET, N.NUMBER,N.PLACE,');
      add('       N.ZIPCODE, N.VATNUMBER, L.COUNTRYNAME, B.VATVALUE');
      add(' FROM INVOICELINES FR');
      add(' INNER JOIN INVOICES F ON FR.INVOICENUMBER = F.INVOICENUMBER');
      add(' LEFT OUTER JOIN CUSTOMERS N ON F.CUSTOMERID = N.ID');
      add(' LEFT OUTER JOIN COUNTRIES L on N.COUNTRYID = L.ID');
      add(' LEFT OUTER JOIN VAT B ON FR.VATCODE = B.CODE');
      add('WHERE F.ID = :id');
    end;
    SQLQuery.Params[0].AsInteger := fid;
    SQLQuery.ReadOnly            := true;
    SQLQuery.Active              := true;
    Transaction.CommitRetaining;
    if SQLQuery.RecordCount > 0 then
    begin
      while not SQLQuery.EOF do
      begin
        Matrix.NewRecord;
        Matrix.SaveField(SQLQuery.FieldByName('ID').Asinteger,'ID',1);
        Matrix.SaveField(SQLQuery.FieldByName('INVOICENUMBER').AsInteger,'INVOICENUMBER',1);
        Matrix.SaveField(SQLQuery.FieldByName('INVOICEDATE').AsDatetime,'INVOICEDATE',1);
        Matrix.SaveField(SQLQuery.FieldByName('DESCRIPTION').AsString,'DESCRIPTION',1);
        Matrix.SaveField(SQLQuery.FieldByName('INVOICEAMOUNT').AsFloat,'AMOUNT',1);
        Matrix.SaveField(SQLQuery.FieldByName('VATAMOUNT').AsFloat,'VATAMOUNT',1);
        Matrix.SaveField(SQLQuery.FieldByName('VATCODE').AsString,'VATCODE',1);
        Matrix.SaveField(SQLQuery.FieldByName('TOTAL').AsFloat,'TOTAL',1);
        Matrix.SaveField(SQLQuery.FieldByName('MEMBER').AsString,'MEMBER',1);
        Matrix.SaveField(SQLQuery.FieldByName('COMPANY').AsString,'COMPANY',1);
        Matrix.SaveField(SQLQuery.FieldByName('ATTENTION').AsString,'ATTENTION',1);
        Matrix.SaveField(SQLQuery.FieldByName('STREET').AsString,'STREET',1);
        Matrix.SaveField(SQLQuery.FieldByName('NUMBER').AsString,'NUMBER',1);
        Matrix.SaveField(SQLQuery.FieldByName('PLACE').AsString,'PLACE',1);
        Matrix.SaveField(SQLQuery.FieldByName('ZIPCODE').AsString,'ZIPCODE',1);
        Matrix.SaveField(SQLQuery.FieldByName('COUNTRYNAME').AsString,'COUNTRYNAME',1);
        Matrix.SaveField(SQLQuery.FieldByName('VATNUMBER').AsString,'VATNUMBER',1);
        Matrix.SaveField(SQLQuery.FieldByName('VATVALUE').AsFloat,'VATVALUE',1);
        Matrix.PostRecord;
        SQLQuery.next;
      end;
    end;
    SQLQuery.active     := false;
  finally
    SQLQuery.Free;
  end;
end;

procedure TInvoicepr.Finish;
var rootfolder : string;
begin
  rootfolder := ExtractFileDir(paramstr(0));
  case fExportKeuze of
    0 : LoadReport(true, rootfolder + '\' + Rapportscript);
    1 : LoadReport(false, rootfolder + '\' + Rapportscript);
  end;
end;

end.

